@@ -26,9 +26,9 @@ LOG = utils.get_logger(__name__)
 
 @six.add_metaclass(abc.ABCMeta)
 class BaseGenericPlugin(base.BaseIdentityPlugin):
-    """An identity plugin that is not version dependant.
+    """An identity plugin that is not version dependent.
 
-    Internally we will construct a version dependant plugin with the resolved
+    Internally we will construct a version dependent plugin with the resolved
     URL and then proxy all calls from the base plugin to the versioned one.
     """
 
@@ -41,10 +41,13 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
                  project_domain_name=None,
                  domain_id=None,
                  domain_name=None,
+                 system_scope=None,
                  trust_id=None,
                  default_domain_id=None,
-                 default_domain_name=None):
-        super(BaseGenericPlugin, self).__init__(auth_url=auth_url)
+                 default_domain_name=None,
+                 reauthenticate=True):
+        super(BaseGenericPlugin, self).__init__(auth_url=auth_url,
+                                                reauthenticate=reauthenticate)
 
         self._project_id = project_id or tenant_id
         self._project_name = project_name or tenant_name
@@ -52,6 +55,7 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
         self._project_domain_name = project_domain_name
         self._domain_id = domain_id
         self._domain_name = domain_name
+        self._system_scope = system_scope
         self._trust_id = trust_id
         self._default_domain_id = default_domain_id
         self._default_domain_name = default_domain_name
@@ -93,21 +97,37 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
         """Return the parameters that are common to v2 plugins."""
         return {'trust_id': self._trust_id,
                 'tenant_id': self._project_id,
-                'tenant_name': self._project_name}
+                'tenant_name': self._project_name,
+                'reauthenticate': self.reauthenticate}
 
     @property
     def _v3_params(self):
         """Return the parameters that are common to v3 plugins."""
-        pr_domain_id = self._project_domain_id or self._default_domain_id
-        pr_domain_name = self._project_domain_name or self._default_domain_name
-
         return {'trust_id': self._trust_id,
+                'system_scope': self._system_scope,
                 'project_id': self._project_id,
                 'project_name': self._project_name,
-                'project_domain_id': pr_domain_id,
-                'project_domain_name': pr_domain_name,
+                'project_domain_id': self.project_domain_id,
+                'project_domain_name': self.project_domain_name,
                 'domain_id': self._domain_id,
-                'domain_name': self._domain_name}
+                'domain_name': self._domain_name,
+                'reauthenticate': self.reauthenticate}
+
+    @property
+    def project_domain_id(self):
+        return self._project_domain_id or self._default_domain_id
+
+    @project_domain_id.setter
+    def project_domain_id(self, value):
+        self._project_domain_id = value
+
+    @property
+    def project_domain_name(self):
+        return self._project_domain_name or self._default_domain_name
+
+    @project_domain_name.setter
+    def project_domain_name(self, value):
+        self._project_domain_name = value
 
     def _do_create_plugin(self, session):
         plugin = None
@@ -118,10 +138,11 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
                                       authenticated=False)
         except (exceptions.DiscoveryFailure,
                 exceptions.HttpError,
-                exceptions.ConnectionError):
-            LOG.warning('Discovering versions from the identity service '
-                        'failed when creating the password plugin. '
-                        'Attempting to determine version from URL.')
+                exceptions.SSLError,
+                exceptions.ConnectionError) as e:
+            LOG.warning('Failed to discover available identity versions when '
+                        'contacting %s. Attempting to parse version from URL.',
+                        self.auth_url)
 
             url_parts = urlparse.urlparse(self.auth_url)
             path = url_parts.path.lower()
@@ -133,6 +154,11 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
                 plugin = self.create_plugin(session, (2, 0), self.auth_url)
             elif path.startswith('/v3'):
                 plugin = self.create_plugin(session, (3, 0), self.auth_url)
+            else:
+                raise exceptions.DiscoveryFailure(
+                    'Could not find versioned identity endpoints when '
+                    'attempting to authenticate. Please check that your '
+                    'auth_url is correct. %s' % e)
 
         else:
             # NOTE(jamielennox): version_data is always in oldest to newest
@@ -171,11 +197,29 @@ class BaseGenericPlugin(base.BaseIdentityPlugin):
             return plugin
 
         # so there were no URLs that i could use for auth of any version.
-        raise exceptions.DiscoveryFailure('Could not determine a suitable URL '
-                                          'for the plugin')
+        raise exceptions.DiscoveryFailure(
+            'Could not find versioned identity endpoints when attempting '
+            'to authenticate. Please check that your auth_url is correct.')
 
     def get_auth_ref(self, session, **kwargs):
         if not self._plugin:
             self._plugin = self._do_create_plugin(session)
 
         return self._plugin.get_auth_ref(session, **kwargs)
+
+    def get_cache_id_elements(self, _implemented=False):
+        # NOTE(jamielennox): implemented here is just a way to make sure that
+        # something overrides this method. We don't want the base
+        # implementation to respond with a dict without the subclass modifying
+        # it to add their own data in case the subclass doesn't support caching
+        if not _implemented:
+            raise NotImplemented()
+
+        return {'auth_url': self.auth_url,
+                'project_id': self._project_id,
+                'project_name': self._project_name,
+                'project_domain_id': self.project_domain_id,
+                'project_domain_name': self.project_domain_name,
+                'domain_id': self._domain_id,
+                'domain_name': self._domain_name,
+                'trust_id': self._trust_id}
